;;; A patch work with code from:
;;;
;;; http://sourceforge.net/mailarchive/message.php?msg_id=6301700
;;; http://www.scsh.net/resources/sunet.html
;;; http://www.scsh.net/cgi-bin/wiki.cgi?SendXmlSuspend

;;; Emilio Lopes <eclig@gmx.net>, 2006-03-17.

(define attribute-rule
  `(@ 
    ((*default*                         ; local override for attributes
      . ,(lambda (attr-key . value)
           (if (null? value)
               (list #\space attr-key "='" attr-key "'")
               (list #\space attr-key "='" value #\')))))
    . ,(lambda (trigger . value) (cons '@  value))))

(define (format-element tag attribs contents)
  (list "<" tag attribs
        (if (pair? contents)
            (list  ">" contents "</" tag ">")
            "/>")
        #\newline))

;; Predicate for attributes in sxml, from SUNet.
(define (sxml-attribute? thing)
  (and (pair? thing)
       (eq? '@ (car thing))))

;; Returns the attribute list after the initial '@. For our
;; convenience, it ignores values that are not sxml-attributes.
;; Also from SUNet.
(define (sxml-attribute-attributes thing)
  (if (sxml-attribute? thing)
      (cdr thing)
      '()))

(define default-rule
  `(*default*
    . ,(lambda (tag . elems)
         (if (and (pair? elems)
                  (sxml-attribute? (car elems)))
             (format-element tag (sxml-attribute-attributes (car elems)) (cdr elems))
             (format-element tag '() elems)))))

(define text-rule
  `(*text*
    . ,(lambda (trigger str) 
	 (if (string? str) (string->goodHTML str) str))))

(define verbatim-rule
  `(*VERBATIM* *preorder*
               . ,(lambda (tag . text) text)))

(define processing-instruction-rule
  `(*PI* *preorder*
         . ,(lambda (tag . elems)
              `(,(string-append "<?" (symbol->string (car elems)) " ")
                ,@(cdr elems)
                "?>"
                #\newline))))

(define top-rule
  `(*TOP*
    . ,(lambda (tag . elems)
         (list elems))))

(define xml-rules
  `(,attribute-rule
    ,default-rule
    ,text-rule
    ,verbatim-rule
    ,processing-instruction-rule
    ,top-rule))

;; From SUNet:
;; adapted from Oleg's SXML-tree-trans.scm SRV:send-reply
;; extended by port argument.
;; Displays low-level-sxml on the port. Low-level-sxml contains only
;; strings, characters and thunks. '() and #f are ignored.
(define (display-low-level-sxml fragments port)
  (let loop ((fragments fragments) (result #f))
    (cond
     ((null? fragments) result)
     ((not (car fragments)) (loop (cdr fragments) result))
     ((null? (car fragments)) (loop (cdr fragments) result))
     ((pair? (car fragments))
      (loop (cdr fragments) (loop (car fragments) result)))
     ((procedure? (car fragments))
      ((car fragments))
      (loop (cdr fragments) #t))
     (else
      (display (car fragments) port)
      (loop (cdr fragments) #t)))))

(define (sxml->xml sxml-expr port)
  (display-low-level-sxml
   (pre-post-order sxml-expr xml-rules)
   port))
